import {Gamesystem} from "./Gamesystem";
import {ProductState} from "./ProductState";
import {ProductTransition} from "./ProductTransition";
import {State} from "./State";
import {Transition} from "./Transition";
import {SimpleState} from "./SimpleState";
import {SimpleGamesystem} from "./SimpleGamesystem";
import {GameModel} from "../GameModel";

export class ProductGamesystem extends Gamesystem<ProductState, ProductTransition> {

  innerGamesystems: Gamesystem<State<any>, Transition<any>>[] = [];
  parentGamesystem: ProductGamesystem | undefined

  static constructFromSimpleGamesystem(simpleGamesystem: SimpleGamesystem, gameModel: GameModel) {
    const productGamesystem = new ProductGamesystem(simpleGamesystem.componentName);
    const parentGamesystem = simpleGamesystem.parentGamesystem;

    if(simpleGamesystem.states.length > 0) {
      simpleGamesystem.componentName += "(Child)";
      productGamesystem.addChildGamesystem(simpleGamesystem);
    }


    if(parentGamesystem != undefined) {
      parentGamesystem.removeChildGamesystem(simpleGamesystem);
      parentGamesystem.addChildGamesystem(productGamesystem);
    } else {
      gameModel.removeGamesystem(simpleGamesystem);
      gameModel.addGamesystem(productGamesystem);
    }

    return productGamesystem;
  }

  createState(label: string, description: string): ProductState | undefined {
    return undefined;
  }

  createTransition(startingState: ProductState, endingState: ProductState): ProductTransition | undefined {
    return undefined;
  }

  removeState(state: ProductState): boolean {
    return false;
  }

  removeInnerState(state: SimpleState) {

  }


  addChildGamesystem(gamesystem: Gamesystem<State<any>, Transition<any>>) {
    this.innerGamesystems.push(gamesystem);
  }

  private removeChildGamesystem(gamesystem: Gamesystem<State<any>, Transition<any>>) {
    this.innerGamesystems = this.innerGamesystems.filter(childSystem => childSystem != gamesystem);
  }
}
